import os, ConfigParser

@plugin
class Auth(object):
    def __init__(self, server):
        self.server = server
        self.commands = [ "addadmin", "addmod", "deladmin", "delmod", "admins", "mods", "owners", "passwd", "identify" ]
        self.owners = {}
        self.admins = {}
        self.mods = {}
        self.server.handle("command", self.handle_command, self.commands)
        self.authdir = self.server.config["confdir"] + "network-auth-users/"
        self.authfile = self.authdir + self.server.config["network"] + "-auth.cfg"
        self._loadusers()
        
    def handle_command(self, channel, user, cmd, args):
        if cmd == "owners":
            self.server.doMessage(channel, user+": My owners are: "+" ".join(self.owners))
        elif cmd == "admins":
            self.server.doMessage(channel, user+": My administrators are: "+" ".join(self.admins))
        elif cmd == "mods":
            self.server.doMessage(channel, user+": My moderators are: "+" ".join(self.mods))
        elif cmd == "passwd":
            if channel == user:
                if user in self.owners.keys():
                    if self.owners[user][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.owners[user][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.owners[user][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                elif user in self.admins.keys():
                    if self.admins[user][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.admins[user][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.admins[user][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                elif user in self.mods.keys():
                    if self.mods[user][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.mods[user][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.mods[user][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                else:
                    self.server.doMessage(channel, user+": You are not in the owners/admins/mods lists.")
                    return
            else:
                self.server.doMessage(channel, user+": Please do that using private messaging.")
                return
        elif cmd == "identify":
            if len(args) < 1:
                self.server.doMessage(channel, user+": Not enough arguments.")
                return
            if channel == user:
                if user in self.owners.keys():
                    if self.owners[user][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.owners[user][0]:
                        self.owners[user][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
                elif user in self.admins.keys():
                    if self.admins[user][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.admins[user][0]:
                        self.admins[user][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
                elif user in self.mods.keys():
                    if self.mods[user][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.mods[user][0]:
                        self.mods[user][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
            else:
                self.server.doMessage(channel, user+": Please do that using private messaging.")
                return
            
        elif self.isOwner(user):
            if len(args) < 1:
                self.server.doMessage(channel, user+": Not enough arguments.")
                return
            
            if cmd == "addadmin":
                if args[0] in self.admins.keys(): return
                self.admins[args[0]] = ['', False]
            elif cmd == "addmod":
                if args[0] in self.mods.keys(): return
                self.mods[args[0]] = ['', False]
            elif cmd == "deladmin":
                if args[0] in self.admins.keys():
                    del self.admins[args[0]]
            elif cmd == "delmod":
                if args[0] in self.mods.keys():
                    del self.mods[args[0]]
    
    def _saveusers(self):
        if not os.path.isdir(self.authdir):
            os.mkdir(self.authdir)
            
        fh = open(self.authfile, "w")
        self.configParser.write(fh)
        fh.close()
        
    __del__ = _saveusers
    
    def _loadusers(self):
        if os.path.isfile(self.authfile):
            self.configParser = ConfigParser.RawConfigParser()
            self.configParser.read(self.authfile)
                
            for section in self.configParser.sections():
                if section == "owners":
                    for user,pswd in self.configParser.items(section):
                        self.owners[user] = [pswd, pswd == '']
                elif section == "admins":
                    for user,pswd in self.configParser.items(section):
                        self.admins[user] = [pswd, pswd == '']
                elif section == "mods":
                    for user,pswd in self.configParser.items(section):
                        self.mods[user] = [pswd, pswd == '']
            
        onick = self.server.config["owner_nick"]
        if not onick in self.owners.keys():
            if getattr(self, "configParser", None) is None:
                self.configParser = ConfigParser.RawConfigParser()
                self.configParser.add_section("owners")
                self.configParser.add_section("admins")
                self.configParser.add_section("mods")
            
            self.configParser.set("owners", onick, '')
            self.owners[onick] = ['', True]
    
    def isOwner(self, user):
        return user in self.owners and self.owners[user][1]
    
    def isAdmin(self, user):
        if user in self.admins and self.admins[user][1]: 
            return True
        else:
            return self.isOwner(user)
    
    def isMod(self, user):
        if user in self.mods and self.mods[user][1]: 
            return True
        elif self.isAdmin(user): 
            return True
        else:
            return self.isOwner(user)
            
